@@ -18,6 +18,7 @@
 package org.apache.commons.compress.archivers.sevenz;
 
 import java.io.ByteArrayOutputStream;
+import java.io.Closeable;
 import java.io.DataOutput;
 import java.io.DataOutputStream;
 import java.io.File;
@@ -26,8 +27,12 @@ import java.io.OutputStream;
 import java.io.RandomAccessFile;
 import java.util.ArrayList;
 import java.util.BitSet;
+import java.util.Collections;
 import java.util.Date;
+import java.util.HashMap;
 import java.util.List;
+import java.util.LinkedList;
+import java.util.Map;
 import java.util.zip.CRC32;
 
 import org.apache.commons.compress.archivers.ArchiveEntry;
@@ -37,16 +42,19 @@ import org.apache.commons.compress.utils.CountingOutputStream;
  * Writes a 7z file.
  * @since 1.6
  */
-public class SevenZOutputFile {
+public class SevenZOutputFile implements Closeable {
     private final RandomAccessFile file;
     private final List<SevenZArchiveEntry> files = new ArrayList<SevenZArchiveEntry>();
     private int numNonEmptyStreams = 0;
-    private CRC32 crc32 = new CRC32();
-    private CRC32 compressedCrc32 = new CRC32();
+    private final CRC32 crc32 = new CRC32();
+    private final CRC32 compressedCrc32 = new CRC32();
     private long fileBytesWritten = 0;
     private boolean finished = false;
     private CountingOutputStream currentOutputStream;
-    private SevenZMethod contentCompression = SevenZMethod.LZMA2;
+    private CountingOutputStream[] additionalCountingStreams;
+    private Iterable<? extends SevenZMethodConfiguration> contentMethods =
+            Collections.singletonList(new SevenZMethodConfiguration(SevenZMethod.LZMA2));
+    private final Map<SevenZArchiveEntry, long[]> additionalSizes = new HashMap<SevenZArchiveEntry, long[]>();
     
     /**
      * Opens file to write a 7z archive to.
@@ -60,15 +68,35 @@ public class SevenZOutputFile {
     }
     
     /**
-     * Sets the compression method to use for entry contents - the
+     * Sets the default compression method to use for entry contents - the
      * default is LZMA2.
      *
      * <p>Currently only {@link SevenZMethod#COPY}, {@link
      * SevenZMethod#LZMA2}, {@link SevenZMethod#BZIP2} and {@link
      * SevenZMethod#DEFLATE} are supported.</p>
+     *
+     * <p>This is a short form for passing a single-element iterable
+     * to {@link #setContentMethods}.</p>
      */
     public void setContentCompression(SevenZMethod method) {
-        this.contentCompression = method;
+        setContentMethods(Collections.singletonList(new SevenZMethodConfiguration(method)));
+    }
+
+    /**
+     * Sets the default (compression) methods to use for entry contents - the
+     * default is LZMA2.
+     *
+     * <p>Currently only {@link SevenZMethod#COPY}, {@link
+     * SevenZMethod#LZMA2}, {@link SevenZMethod#BZIP2} and {@link
+     * SevenZMethod#DEFLATE} are supported.</p>
+     *
+     * <p>The methods will be consulted in iteration order to create
+     * the final output.</p>
+     *
+     * @since 1.8
+     */
+    public void setContentMethods(Iterable<? extends SevenZMethodConfiguration> methods) {
+        this.contentMethods = reverse(methods);
     }
 
     /**
@@ -131,9 +159,16 @@ public class SevenZOutputFile {
             ++numNonEmptyStreams;
             entry.setSize(currentOutputStream.getBytesWritten());
             entry.setCompressedSize(fileBytesWritten);
-            entry.setCrc((int) crc32.getValue());
-            entry.setCompressedCrc((int) compressedCrc32.getValue());
+            entry.setCrcValue(crc32.getValue());
+            entry.setCompressedCrcValue(compressedCrc32.getValue());
             entry.setHasCrc(true);
+            if (additionalCountingStreams != null) {
+                long[] sizes = new long[additionalCountingStreams.length];
+                for (int i = 0; i < additionalCountingStreams.length; i++) {
+                    sizes[i] = additionalCountingStreams[i].getBytesWritten();
+                }
+                additionalSizes.put(entry, sizes);
+            }
         } else {
             entry.setHasStream(false);
             entry.setSize(0);
@@ -141,11 +176,12 @@ public class SevenZOutputFile {
             entry.setHasCrc(false);
         }
         currentOutputStream = null;
+        additionalCountingStreams = null;
         crc32.reset();
         compressedCrc32.reset();
         fileBytesWritten = 0;
     }
-    
+
     /**
      * Writes a byte to the current archive entry.
      * @param b The byte to be written.
@@ -219,7 +255,7 @@ public class SevenZOutputFile {
         final byte[] startHeaderBytes = startHeaderBaos.toByteArray();
         crc32.reset();
         crc32.update(startHeaderBytes);
-        file.writeInt(Integer.reverseBytes((int)crc32.getValue()));
+        file.writeInt(Integer.reverseBytes((int) crc32.getValue()));
         file.write(startHeaderBytes);
     }
     
@@ -236,11 +272,26 @@ public class SevenZOutputFile {
     }
 
     private CountingOutputStream setupFileOutputStream() throws IOException {
+        if (files.isEmpty()) {
+            throw new IllegalStateException("No current 7z entry");
+        }
+
         OutputStream out = new OutputStreamWrapper();
-        return new CountingOutputStream(Coders
-                                        .addEncoder(out,
-                                                    contentCompression,
-                                                    null)) {
+        ArrayList<CountingOutputStream> moreStreams = new ArrayList<CountingOutputStream>();
+        boolean first = true;
+        for (SevenZMethodConfiguration m : getContentMethods(files.get(files.size() - 1))) {
+            if (!first) {
+                CountingOutputStream cos = new CountingOutputStream(out);
+                moreStreams.add(cos);
+                out = cos;
+            }
+            out = Coders.addEncoder(out, m.getMethod(), m.getOptions());
+            first = false;
+        }
+        if (!moreStreams.isEmpty()) {
+            additionalCountingStreams = moreStreams.toArray(new CountingOutputStream[moreStreams.size()]);
+        }
+        return new CountingOutputStream(out) {
             @Override
             public void write(final int b) throws IOException {
                 super.write(b);
@@ -262,6 +313,11 @@ public class SevenZOutputFile {
         };
     }
 
+    private Iterable<? extends SevenZMethodConfiguration> getContentMethods(SevenZArchiveEntry entry) {
+        Iterable<? extends SevenZMethodConfiguration> ms = entry.getContentMethods();
+        return ms == null ? contentMethods : ms;
+    }
+
     private void writeHeader(final DataOutput header) throws IOException {
         header.write(NID.kHeader);
         
@@ -296,10 +352,10 @@ public class SevenZOutputFile {
         }
         
         header.write(NID.kCRC);
-        header.write(1);
+        header.write(1); // "allAreDefined" == true
         for (final SevenZArchiveEntry entry : files) {
             if (entry.hasStream()) {
-                header.writeInt(Integer.reverseBytes(entry.getCompressedCrc()));
+                header.writeInt(Integer.reverseBytes((int) entry.getCompressedCrcValue()));
             }
         }
         
@@ -312,44 +368,67 @@ public class SevenZOutputFile {
         header.write(NID.kFolder);
         writeUint64(header, numNonEmptyStreams);
         header.write(0);
-        for (int i = 0; i < numNonEmptyStreams; i++) {
-            writeFolder(header);
+        for (SevenZArchiveEntry entry : files) {
+            if (entry.hasStream()) {
+                writeFolder(header, entry);
+            }
         }
-        
+
         header.write(NID.kCodersUnpackSize);
         for (final SevenZArchiveEntry entry : files) {
             if (entry.hasStream()) {
+                long[] moreSizes = additionalSizes.get(entry);
+                if (moreSizes != null) {
+                    for (long s : moreSizes) {
+                        writeUint64(header, s);
+                    }
+                }
                 writeUint64(header, entry.getSize());
             }
         }
         
         header.write(NID.kCRC);
-        header.write(1);
+        header.write(1); // "allAreDefined" == true
         for (final SevenZArchiveEntry entry : files) {
             if (entry.hasStream()) {
-                header.writeInt(Integer.reverseBytes(entry.getCrc()));
+                header.writeInt(Integer.reverseBytes((int) entry.getCrcValue()));
             }
         }
         
         header.write(NID.kEnd);
     }
     
-    private void writeFolder(final DataOutput header) throws IOException {
-        // one coder
-        writeUint64(header, 1);
-        byte[] id = contentCompression.getId();
-        byte[] properties = contentCompression.getProperties();
+    private void writeFolder(final DataOutput header, SevenZArchiveEntry entry) throws IOException {
+        ByteArrayOutputStream bos = new ByteArrayOutputStream();
+        int numCoders = 0;
+        for (SevenZMethodConfiguration m : getContentMethods(entry)) {
+            numCoders++;
+            writeSingleCodec(m, bos);
+        }
+
+        writeUint64(header, numCoders);
+        header.write(bos.toByteArray());
+        for (int i = 0; i < numCoders - 1; i++) {
+            writeUint64(header, i + 1);
+            writeUint64(header, i);
+        }
+    }
+
+    private void writeSingleCodec(SevenZMethodConfiguration m, OutputStream bos) throws IOException {
+        byte[] id = m.getMethod().getId();
+        byte[] properties = Coders.findByMethod(m.getMethod())
+            .getOptionsAsProperties(m.getOptions());
 
         int codecFlags = id.length;
         if (properties.length > 0) {
             codecFlags |= 0x20;
         }
-        header.write(codecFlags);
-        header.write(id);
+        bos.write(codecFlags);
+        bos.write(id);
 
         if (properties.length > 0) {
-            header.write(properties.length);
-            header.write(properties);
+            bos.write(properties.length);
+            bos.write(properties);
         }
     }
     
@@ -380,7 +459,7 @@ public class SevenZOutputFile {
         writeFileATimes(header);
         writeFileMTimes(header);
         writeFileWindowsAttributes(header);
-        header.write(0);
+        header.write(NID.kEnd);
     }
     
     private void writeFileEmptyStreams(final DataOutput header) throws IOException {
@@ -489,7 +568,7 @@ public class SevenZOutputFile {
                 }
                 writeBits(out, cTimes, files.size());
             } else {
-                out.write(1);
+                out.write(1); // "allAreDefined" == true
             }
             out.write(0);
             for (final SevenZArchiveEntry entry : files) {
@@ -525,7 +604,7 @@ public class SevenZOutputFile {
                 }
                 writeBits(out, aTimes, files.size());
             } else {
-                out.write(1);
+                out.write(1); // "allAreDefined" == true
             }
             out.write(0);
             for (final SevenZArchiveEntry entry : files) {
@@ -561,7 +640,7 @@ public class SevenZOutputFile {
                 }
                 writeBits(out, mTimes, files.size());
             } else {
-                out.write(1);
+                out.write(1); // "allAreDefined" == true
             }
             out.write(0);
             for (final SevenZArchiveEntry entry : files) {
@@ -597,7 +676,7 @@ public class SevenZOutputFile {
                 }
                 writeBits(out, attributes, files.size());
             } else {
-                out.write(1);
+                out.write(1); // "allAreDefined" == true
             }
             out.write(0);
             for (final SevenZArchiveEntry entry : files) {
@@ -636,18 +715,25 @@ public class SevenZOutputFile {
         int shift = 7;
         for (int i = 0; i < length; i++) {
             cache |= ((bits.get(i) ? 1 : 0) << shift);
-            --shift;
-            if (shift == 0) {
+            if (--shift < 0) {
                 header.write(cache);
                 shift = 7;
                 cache = 0;
             }
         }
-        if (length > 0 && shift > 0) {
+        if (shift != 7) {
             header.write(cache);
         }
     }
 
+    private static <T> Iterable<T> reverse(Iterable<T> i) {
+        LinkedList<T> l = new LinkedList<T>();
+        for (T t : i) {
+            l.addFirst(t);
+        }
+        return l;
+    }
+
     private class OutputStreamWrapper extends OutputStream {
         @Override
         public void write(final int b) throws IOException {
